import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * This is a class that emulates blocks of Scratch-Sprites as Greenfoot-methods.
 * It also includes explanations of how other Scratch blocks can be transfered to Greenfoot-Syntax.
 * Scratch-Homepage: http://scratch.mit.edu
 * 
 * @author Ralf
 * @version 5.12.09
 */

public class ScratchSprite  extends Actor
{
    private int size;
    private GreenfootImage costume1;
    private GreenfootImage costume2;
    private GreenfootImage costume3;
    private GreenfootImage costume4;
    private GreenfootImage costume5;
    //add more if needed 

    public void act() 
    {
        /** 
         * equivalent to "When Green Flag Clicked" + "forever" control
         * block in scratch
         * 
         * Add method calls here!
         */
    }    
 
// BLOCKS

// MOTION    
    
    /** 
     * go to x: _ y:_ 
     * Greenfoot-Example: goTo (100,50);
     **/
     public void goTo(int x, int y) {
         setLocation(x, y);
     }

     /**
      * change x by _ dots
      * Greenfoot-Example: changeXby (10);
      **/
      public void changeXby (int dots) {
          setLocation(getX()+dots, getY());
      }

      /**
      * change y by _ dots
      * Greenfoot-Example: changeYby (10);
      **/
      public void changeYby (int dots) {
          setLocation(getX(), getY()+dots);
      }
      
      /**
       * move _ steps
       * Greenfoot-Example: move (10)
       **/
        public void move(double steps) {
            double angle = Math.toRadians( getRotation() );
            int x = (int) Math.round(getX() + Math.cos(angle) * steps);
            int y = (int) Math.round(getY() + Math.sin(angle) * steps);      
            setLocation(x, y);        
        }

        /**
         *  turn _ degrees
         *  Greenfoot-Example: turn (90);
         **/
         public void turn(int angle)
         {
             setRotation(getRotation() + angle);
         }
    
         /**
          * point in direction _
          * Greenfoot-Example: pointInDirection (90);
          **/
          public void pointInDirection(int direction) {
              setRotation (direction);
          }
 
          /**
           *   if on edge, bounce
           *   Greenfoot-Example: ifOnEdgeBounce();
           **/
           public void ifOnEdgeBounce() {   // edge minus 1 dots in all directions
               int x = getX();
               int y = getY();
               if(x >= (getWorld().getWidth() - 1)) {
                   setRotation (180 - getRotation());
               }
               if(x < 1) {
                   setRotation (180-getRotation());
               }
               if(y >= getWorld().getHeight() - 1) {
                   setRotation (360-getRotation());
               }
               if(y < 1) {
                   setRotation (360-getRotation());
               }
               setLocation(x, y);
            }
    
/*  Looks:
    Costumes:

    Easy: just one costume:
          The costume is assigned by a right-click on the class: Set image

    Advanced: 
          If you have more than one costume the costumes need to be declared as separate attributes in the class, e.g.:

          public class Crab extends Animal
          {
              private GreenfootImage image1;
              private GreenfootImage image2;
          }
    
          Class ScratchSprite owns right now 5 costumes:
          private GreenfootImage costume1;
          private GreenfootImage costume2;
          private GreenfootImage costume3;
          private GreenfootImage costume4;
          private GreenfootImage costume5;
 */ 

    /**
     * Switch to costume _ 
     * Greenfoot-Example: switchToCostume (image1);
     * 
     */
     public void switchToCostume(GreenfootImage image1) {
        setImage(image1);
     }

    /**
     *  next costume (Only toggles costume 1 and 2 yet). 
     *  Add more, if needed.
     *  Greenfoot-Example: nextCostume();
     **/
     public void nextCostume() {
        if (getImage() == costume1) 
        {
            setImage(costume2);
        }
        else
        {
            setImage(costume1);
        }
     }

// Size
     /**
      * set size to n
      * Greenfoot-Example: setSize (50);
      **/
      public void setSize(int size)
      {
        this.size = size;
        GreenfootImage image = getImage();
        image.scale(size, size);
      }

      /*
      * go to front
      * 
      * setPaintOrder //usually called from within the constructor class, e.g.
      * setPaintOrder(Ant.class, Counter.class, Food.class);
      * 
      * To set within an Actor class  write real class names instead of Myactor.class etc.
      * and implement the method as follows:
      * 
      * public void goToFront() {
      *       getWorld().setPaintOrder(Myactor.class, MyOtherActor.class); // Don't forget to adjust the real class names!
      * }
      */

// CONTROL  - Only for refence purpose. Copy the code if needed.

        /* Forever:
         *
         * Write commands in Act-method. It will be repeated forever when Button RUN is clicked.
         */

        /* if 
         * 
         * if (condition) {
         *        statements
         * }
         */


        /* if - else
         * 
         * if (condition) {
         *         statements
         * }
         * else {
         *     other statements
         * }
         */  

        
        /* repeat 10 times
         * 
         * for (int i = 0; i < 10; i++) {
         *        statements
         * }
         */

         /* forever if
          * 
          * while (condition)  {
          *     statements
          * }
          */

          /* broadcast ____
           * use the direct call of object and method:
           * ObjectToCall.methodName
           */ 

          /* when ___ Key pressed
           * 
           * Example: in act() call checkKeypress():  
           * 
           * public void checkKeypress()
           * {
           *     if (Greenfoot.isKeyDown("left")) 
           *     {
           *         turn(-4);
           *        }
           *        if (Greenfoot.isKeyDown("right")) 
           *        {
           *            turn(4);
           *        }
           *     }
           */

          /* when Object clicked
           * if(Greenfoot.mouseClicked(this)) {
           *     statements
           * }
           */
          
            
          /**
          * wait  x secs
          * Greenfoot-Example: wait (100);
          * Note: Not really useful in Greenfoot, since all Greenfoot will stop for that time!
          */    
          public void wait (int time) {
             Greenfoot.delay(time);  //stops all Greenfoot for time steps!!!
         }
    
         /*    
          * Better include a boolean "paused" as attribute and implement wait there:
          * public void act(){
          *     if (!isPaused() {
          *         statements
          * }
          */
                   
          /**
           * stop all
           * Greenfoot-Example: stopAll()
           */
           public void stopAll() {
               Greenfoot.stop() ; // stops the scenario
           }
           
// SENSING
        /**
        * touching _ ?
        * Greenfoot-Example: touching (otherCat)
        * Note: Objects needs to be handed over as Parameter (in Greenfoot we dont know 
        *       the name of the object!)
        *       Check example below!
        */

       public boolean touching (ScratchSprite object) {
           ScratchSprite touch = (ScratchSprite) getOneIntersectingObject(ScratchSprite.class);
           return (touch == object) ;
       }

        /* In Scratch
         * If touching Bar1
         *      point in (180 - direction of Ball1)
         *      move 5 steps
         *      
         * Since we don't know the name of Bar 1 in Greenfoot (names only exist internally and are quite cryptic), we have to find out what it is and assign it a name:
         */

        /* 
         * Bar bar1 = (Bar) getOneIntersectingObject(Bar.class); // means: make a new Bar and call it bar1, then find any object touching me (what is an object of Bar.class) and copy it to bar1
         * if (bar1 != null) {                                   // if the object bar1 is not empty, 
         *         pointInDirection (360-getRotation());         // change the direction so that the ball bumps away
         *         move(2);                                      // move away to get away from the bar
         *  }
         */            

//SOUND

    /**
     * play sound _
     * Greenfoot-Example: Greenfoot.playSound("meow.wav");
     * Note: Necesssary to use whole statement including "Greenfoot"
     */
    public void playSound (java.lang.String soundFile) {
        Greenfoot.playSound(soundFile);
    }

// OPERATORS
    /**
    * pick random number min to max
    * Greenfoot-Example: pickRandomNumber (1, 100)
    */

   public int pickRandomNumber (int min, int max) {
       int random = Greenfoot.getRandomNumber (max - min + 1);
       return (random + min);
   }
}